/*	This program compiles using devkitARM_r13-win32 and libnds

	The demo includes the following features:
	-4 scrolling 2D backgrounds (MODE_0_2D)
	-hardware timers
	-extended palettes
	-alpha blended backgrounds
	-controls (including detecting whether or not the ds is open or closed)

	NOTES:	Dualis: Crashes unless you comment out the clock code at the end.
	It runs very strangely in dualis v10 because dualis doesn't support
	extended palettes in tiled modes yet.
	
	The real DS runs this demo flawless when properly compiled. */

#include <nds.h>				//includes many useful libnds headers
#include "level1.h"				//level data (tiles, tilemap and palette)
#include "nds/arm9/console.h"	//a handy printing console


//waitVBlank:
/* 	Wait for vblank causes the cpu to loop until it is safe to modify the graphics
	hardware again.  After one frame is displayed on the LCD, the next one does
	not begin immediately.  Instead there is a blank period of perhaps 150,000
	cycles that gives you plenty of time to setup backgrounds, scroll your tile
	map and play with the oam for sprites (no sprites in this demo). If you modify
	the graphics after the vblank is over (and the next frame has begun) you'll get
	flickering.  Try commenting out the vblank loops and looping until DISP_Y == 96 */
void waitVBlank()
{
	/* This vblank method always exits at the start of line 193 (the first vblank
	line.  You can't simply use while (DISP_Y != 193); because if you call it
	from line 193 it will exit without waiting.  This is undesirable because
	a proper wait for vblank makes an excellent timer (causing your gameloop to
	run precisely 60 times a second) */
	while(DISP_Y != 192); //loops until the last line of the current frame
	while(DISP_Y == 192); //loops until the vblank starts
}


//main: Execution starts here
//
int main()
{
	/* Wait for vblank before we initialize the graphics hardware */
	waitVBlank();
	
	/* Power on the LCD's and 2D cores.  Other options include the 3D core and
	matrix units.  Check the libnds or ndslib arm9/video.h for more options */
	POWER_CR = POWER_LCD_TOP | POWER_2D | POWER_LCD_BOTTOM | POWER_2D_SUB;

	/* Start the hardware timers in a cascading chain (essentially using the 2
	independent 16bit timers as one 32 bit timer).  Timer cascade causes the 
	overflow of the first timer to increment the second timer.  When writing to
	a timer, the write does not take effect until the timer is enabled.  The
	last option, TIMER_DIV can also take 1, 64 and 256.  This is the number of
	clock cycles that elapse before the timer is incremented.  1 is good timing
	algorithms to determine running efficiency, 1024 is good for larger scale
	timing (like a seconds counter for a game with a timer; although you can
	also set up a vblank timer by simply incrementing once per frame) */
	TIMER0_DATA = 0;
	TIMER1_DATA = 0;
	TIMER1_CR = TIMER_ENABLE | TIMER_CASCADE;
	TIMER0_CR = TIMER_ENABLE | TIMER_DIV_1024;

	/* Initialize all vram banks.  There are library functions that make this look
	nicer, but I prefer to work directly with the registers so I know precisely 
	what to expect from them.  Various emulators may not initialize the same way
	the real hardware does so I prefer to manually intialize everything from 0.
	VRAM_A_CR is the control register used to control settings for VRAM A. We enable
	it and assign it to the main background. */
	VRAM_A_CR = VRAM_ENABLE | VRAM_A_MAIN_BG;
	VRAM_B_CR = 0;
	VRAM_C_CR = 0;
	VRAM_D_CR = 0;
	VRAM_E_CR = 0;
	VRAM_F_CR = 0; //F and G will be used for extended palettes but leave them
	VRAM_G_CR = 0; //alone for now.
	VRAM_H_CR = 0;
	VRAM_I_CR = 0;	
	
	/* Set the display mode for each 2D core.  On the main display we're using
	MODE_0_2D.  This allow for 4 ordinay scrolling tiled backgrounds.  Other modes
	allow for giant bitmaps, rotation and scaling backgrounds etc.  The next option
	activates extended palettes.  This will allow each background to access up to 16
	256 color palettes per frame (64 total).  The rest of the options simply enable
	the four backgrounds.  We'll set the SUB_DISPLAY_CR to 0 for now. */
	DISPLAY_CR = MODE_0_2D | DISPLAY_BG_EXT_PALETTE | DISPLAY_BG0_ACTIVE
			   | DISPLAY_BG1_ACTIVE | DISPLAY_BG2_ACTIVE | DISPLAY_BG3_ACTIVE;
	SUB_DISPLAY_CR = 0;
	
	/* When using extended palettes, you still have the option to use 16 color
	palettes.  You can enable this using BG_16_COLOR on any background, and
	conversely to use extended palettes with that background you must set it to
	BG_256_COLOR.  The tile base is the location from which tiles are indexed for
	the background.  In this demo all the backgrounds are using the same tiles
	so we'll give them all the same base.  The map base is the location where
	the tilemap for each background is stored.  Each BG could also use the same
	map base, but I've separated them so they can all use different extended
	palettes.  The BG_32x64 specifier indicates the size of the tilemap.  Other
	options are 32x32, 64x32 and 64x64.  Finally, the extended palette slot
	is set for BG0 and BG1.  A slot contains 16 256 color palettes.  BG0 and BG1
	can instantly change slots by changing its BG _CR, but BG0 can only use slot
	0 and 2, and BG1 can only use slot 1 and 3.  BG2 and BG3 are forced to use
	slots 2 and 3 respectively.  In this demo we'll give each BG its own slot */
	BG0_CR = BG_256_COLOR | BG_TILE_BASE(1) | BG_MAP_BASE(0) | BG_32x64 | BG_PALETTE_SLOT0;
	BG1_CR = BG_256_COLOR | BG_TILE_BASE(1) | BG_MAP_BASE(2) | BG_32x64 | BG_PALETTE_SLOT1;
	BG2_CR = BG_256_COLOR | BG_TILE_BASE(1) | BG_MAP_BASE(4) | BG_32x64; //SLOT 2
	BG3_CR = BG_256_COLOR | BG_TILE_BASE(1) | BG_MAP_BASE(6) | BG_32x64; //SLOT 3

	/* Point to the start of tile memory.  In a more advanced program, each
	background would probably have it's own tile pointer.  The argument of
	BG_TILE_RAM should be identical to the BG_TILE_BASE set above.  The offset
	indicates which 16KB block to start from.  You can set it from 0 to 31
	(although each VRAM bank you map for tiles can only make use of 0 to 7,
	8 to 15, 16 to 23 or 24 to 31. */
	u16* tile = (u16*)BG_TILE_RAM(1);

	/* These entries should correspond to the BG_MAP_BASE set above.  Each
	argument specifies the 2KB offset from the start of the ram you assigned
	to MAIN_BG.  a 32x32 map require 2KB and a 64x64 requires 8KB.  We're using
	32x64 maps at 4KB each.  For a real game you'll want to stream new tilemap
	data into video memory as the screen scrolls.  If you don't then 64x64 tiles
	is the biggest "level" you can have (and that's only 2x3 screens or so). 
	This demo does not use dynamic tilemap streaming.  Tilemaps automatically
	wrap if you try to display past their edges (this helps with streaming). */
	u16* map0 = (u16*)BG_MAP_RAM(0);
	u16* map1 = (u16*)BG_MAP_RAM(2);
	u16* map2 = (u16*)BG_MAP_RAM(4);
	u16* map3 = (u16*)BG_MAP_RAM(6);

	/* Copy tiles into video memory starting at the first 16KB boundary (we can't
	start at 0 because the tilemaps are going in the first 16KB).  L1GFX is an
	array that is defined in level1.h.  It was generated using a tool (pcxtogba). */
	for (u16 i = 0; i < sizeof(L1GFX)/sizeof(u16); i++)	tile[i] = L1GFX[i];

	/* I'll be using the proper palette that goes with this level data to generate
	64 palettes.  Each palette will just be various darker shades of the original
	averaged into one color channel or two (red, green, blue, yellow etc.).  To
	easily access the color components of this palette, I'm using the following
	macros.  %256 ensures that the index used is always from 0 to 255.  The shift
	moves the desired bits to the the lowest position and the hex masks out the
	remaining data leaving us with the color we need.  L1PAL is also in level.h */
	#define RED		(L1PAL[i%256] & 0x001F)			
	#define GREEN	((L1PAL[i%256]>>5) & 0x001F)	
	#define BLUE	((L1PAL[i%256]>>10) & 0x001F)	

	/* Extended palettes can be placed into a number of different VRAM banks. 
	Check nds/arm9/vide.h for more details.  We'll be using F and G.  Before you
	can write to these banks, they must be in "LCD" mode (something of a misnomer) */
	VRAM_F_CR = VRAM_ENABLE | VRAM_F_LCD;
	VRAM_G_CR = VRAM_ENABLE | VRAM_G_LCD;
	
	/* Loop 4096 times (16 palettes with 256 entries each) and write 4 entries
	per iteration. */
	for (u16 i = 0; i < (sizeof(L1PAL)/sizeof(u16))*16; i++) 
	{
		/* For each palette we calculate the greyscale average by adding all of
		the color channels together and dividing by 3 or more.  Each palette will
		be a little darker than the previous which we'll use to create a fade effect.
		A better use of extended palettes would be to have unique art assets that
		use the palettes for art rather than a cheap effect like this. */
		
		//slot 0, red
		VRAM_F[i]		= RGB15((RED+BLUE+GREEN)/((i/256)+3),0,0);
		//slot 1, green
		VRAM_F[i+4096] 	= RGB15(0,(RED+BLUE+GREEN)/((i/256)+3),0);
		//slot 2, blue
		VRAM_G[i] 		= RGB15(0,0,(RED+BLUE+GREEN)/((i/256)+3));
		//slot 3, yellow
		VRAM_G[i+4096] 	= RGB15((RED+BLUE+GREEN)/((i/256)+3),(RED+BLUE+GREEN)/((i/256)+3),0);
	}
	
	/* Now that the palettes have been written, set VRAM_F and G to extended
	palette mode.  The ndslib does not have appropriate defines for this. The first
	line sets VRAM_F to store slots 0 and 1, and the hex on the second line sets
	VRAM_G to store slots 2 and 3. */
	VRAM_F_CR = VRAM_ENABLE | VRAM_F_BG_EXT_PALETTE; 	//slots 0 and 1
	VRAM_G_CR = VRAM_ENABLE | 0xC; 						//slots 2 and 3

	/* Some emulators without extended palette support may still run by reading the
	regular palette, so we'll store the unmodified palette here. */
	for (u16 i = 0; i < (sizeof(L1PAL)/sizeof(u16))*16; i++)
		BG_PALETTE[i] = L1PAL[i];
		
	/* If any of the 4 backgrounds are set to 16_COLOR mode it still uses the
	normal palette.  If only extended palettes are used, then only the first entry
	need be set for the background color. The background color is displayed whenever
	transparency or disabled backgrounds allows you to see past the 4th BG into
	the void. */
	BG_PALETTE[0] = RGB15(6,4,10); //a dark blue with slight grey purple hints

	/* Now that we have tiles and palettes, we still need to copy the tilemaps into
	memory.  Tilemaps reference the tiles and palettes to create the graphics on
	the screen at any given time.  The low 10 bits chooses one of 1024 tiles, the
	next 2 bits specified horizontal and/or vertical flip (not used here) and the
	last 4 bits references one of 16 palettes (from the slot set in the BG*_CR. */
	for (u16 i = 0; i < sizeof(L1MAP)/sizeof(u16); i++)
	{	
		/* Try setting bits 10 and 11 to flip tiles on some backgrounds.  The math
		below gives every two columns a different extended palette creating a fade. */
		map0[i] = L1MAP[i] | (((i%32)/2)<<12);
		map1[i] = L1MAP[i] | (((i%32)/2)<<12);
		map2[i] = L1MAP[i] | (((i%32)/2)<<12);
		map3[i] = L1MAP[i] | (((i%32)/2)<<12);
	}

	/* Each tilemap is 32x64.  We only copied tiles into the first 32x32 block.
	For the second 32x32 block we'll set all the tiles to transparent so that we
	can see through them to the other four backgrounds. */
	for (u16 i = sizeof(L1MAP)/sizeof(u16); i < 2* sizeof(L1MAP)/sizeof(u16); i++)
	{
		map0[i] = 0x0000;
		map1[i] = 0x0000;
		map2[i] = 0x0000;
		map3[i] = 0x0000;
	}

	/* These variables to hold the screen's scroll position (needed because the
	register is write only).  The scroll position is the offset into the tilemap.
	If you design a 64x64 tilemap and display it from 0,0 you'll see 32x24 tiles
	starting in the upper left corner.  To scroll to the right by one tile you'd 
	increment the x_scroll by 8 (one tile is 8x8) and update the scroll register.
	We'll be updating the scroll register in the game loop below. */
	u16 y_scroll = 0;
	u16 x_scroll = 0;

	/* Initialize the blending register to null */
	BLEND_CR = BLEND_NONE;
	
	/* As with scroll, these registers are write only so we'll need variables to
	use as intermediates so that we can read and increment the values.  The use
	of these will be explained in the gameloop below */
	BLEND_AB = 0x0000;
	BLEND_Y  = 0x0000;	
	u16 blend_a = 0;
	u16 blend_b = 0;
	u16 blend_y = 0;	
	u16 blend_mode = 0;
		
	/* WE'll use the sub screen to display a printing console so that we can
	monitor certain variables such as which screens are active and blend rates. */
	videoSetModeSub(MODE_0_2D | DISPLAY_BG0_ACTIVE | BG_16_COLOR);
	vramSetBankC(VRAM_C_SUB_BG);
	SUB_BG0_CR = BG_MAP_BASE(31);
	BG_PALETTE_SUB[255] = RGB15(31,31,31); //text color
	consoleInitDefault((u16*)BG_MAP_RAM_SUB(31), (u16*)BG_TILE_RAM_SUB(0), 16);
	
	/* The game loop.  After initialization, this is where all of the action
	takes place.  In a real game you'd be doing things like moving the player,
	updating entities (bullets, enemies via a.i.) and processing touch screen
	input.  Here's we'll be scrolling our backgrounds in response to button
	presses and changing blend modes. */
	while(1)
	{
		/* wait until the blank period between lcd writes to modify graphics data */
		waitVBlank(); 
		
		/* kill the LCD POWER if the lid is closed, else turn it on */
		static s8 lid_state = -1; //start the lid_state at null
		if (IPC->buttons & IPC_LID_CLOSED)
		{
			if (lid_state != 1) //if state isn't already closed
			{
				lid_state = 1; //change state, disable power
				POWER_CR = 0;
			}
		}
		else
		{
			if (lid_state != 0) //if state isn't already opened
			{
				lid_state = 0; //change state and enable power
				POWER_CR = POWER_LCD_TOP | POWER_2D | POWER_LCD_BOTTOM | POWER_2D_SUB;
			}
		}
		
		/* This reads the keys for use with the functions below.  If you don't call
		this before using the keysHeld or keysDown functions, you'll be using old
		or garbage key information. */
		scanKeys();
		
		/* scroll the bg's while the d-pad is pressed */
		if (keysHeld()&KEY_UP)		y_scroll++;
		if (keysHeld()&KEY_DOWN)	y_scroll--;
		if (keysHeld()&KEY_LEFT)	x_scroll++;		
		if (keysHeld()&KEY_RIGHT)	x_scroll--;		

		/* When a face button is pressed AND released, disable or enable a bg */
		if (keysDown()&KEY_B)
			DISPLAY_CR = (DISPLAY_CR & ~DISPLAY_BG0_ACTIVE) | (~DISPLAY_CR & DISPLAY_BG0_ACTIVE);
		if (keysDown()&KEY_A)
			DISPLAY_CR = (DISPLAY_CR & ~DISPLAY_BG1_ACTIVE) | (~DISPLAY_CR & DISPLAY_BG1_ACTIVE);
		if (keysDown()&KEY_Y)
			DISPLAY_CR = (DISPLAY_CR & ~DISPLAY_BG2_ACTIVE) | (~DISPLAY_CR & DISPLAY_BG2_ACTIVE);
		if (keysDown()&KEY_X)
			DISPLAY_CR = (DISPLAY_CR & ~DISPLAY_BG3_ACTIVE) | (~DISPLAY_CR & DISPLAY_BG3_ACTIVE);
			
		/* Use L, R and Select to cycle the blending coefficients.  Coefficient
		A controls how much of the source contributes to the finel pixel.
		Coefficient B controls how much of the destination contributes to the final
		pixel.  For example, a window should allow a lot of the destinition color
		to show through so blend_b would be set to a high value.  At the same
		time, the window itself would be barely visible so blend_a should be
		set to a low value.  Setting both to 0 results in black.  Setting both
		to 31 results in a ghost like image where each can be seen equally well */
		if (keysHeld()&KEY_L) blend_a++;
		if (blend_a > 31) blend_a = 0;
		if (keysHeld()&KEY_R) blend_b++;
		if (blend_b > 31) blend_b = 0;		
		
		/* This sets the blending for the FADE_WHITE and FADE_BLACK modes. (0
		is normal 31 is fully black or white). This is good for just what it
		sounds like.  If you want to do a fade out to white or black use this. */
		if (keysHeld()&KEY_SELECT)	blend_y++;		
		if (blend_y > 31) blend_y = 0;
		
		/* The blend mode simply selects between no blending, alpha blending
		and fade white or black.  That's four modes.  We're also setting the
		blending sources and destinitions below.  The source is the bg you want
		to blend (i.e. the window) and the destinition is the stuff you want
		it to blend with (the scenery outside the window).  You can't blend 3
		backgrounds together.  For example, if you disable bg1 in this demo
		you'll see that although bg0 and bg2 are transparent and blend together
		you cannot see bg3 through the two of them together. */
		if (keysDown()&KEY_START)
		{
			blend_mode++;
			if (blend_mode>3) blend_mode = 0;
			
			BLEND_CR = (blend_mode<<6) | BLEND_SRC_BG0 | BLEND_SRC_BG2
			| BLEND_DST_BG0 | BLEND_DST_BG1 | BLEND_DST_BG2 | BLEND_DST_BG3;
		}
		
		/* Now we can write our blend coefficients to the write only registers */
		BLEND_AB = blend_a | (blend_b<<8);
		BLEND_Y = blend_y;			
			
		/* The BG scroll registers are also write only.  We'll scroll the higher
		backgrounds at a faster rate.  4 pixels per frame is quite fast but still
		looks good. */
		BG0_X0 = x_scroll*4;
		BG0_Y0 = y_scroll*4;
		BG1_X0 = x_scroll*3;
		BG1_Y0 = y_scroll*3;
		BG2_X0 = x_scroll*2;
		BG2_Y0 = y_scroll*2;
		BG3_X0 = x_scroll*1;
		BG3_Y0 = y_scroll*1;
		
		/* Finally, we'll write various data to the console */
		consolePrintSet(0,0);
		consolePrintf("Ext Palette Demo by rize_foa\n\n");
		
		/* The two timers set up above are good for about 36 hours before overflowing
		and rolling back to 0.  If we used a faster update mode than 1024 it would over
		flow 4, 16 or 1024 times as quickly. */
		u32 sec = (TIMER1_DATA*2) + (TIMER0_DATA/32768);
		u32 min = sec/60;
		u32 hrs = min/60;
		sec = sec%60;
		min = min%60;
		consolePrintf("Running Time: %d:%d:%d  \n\n", hrs, min, sec);		
		
		/* Shows which backgrounds are active */
		consolePrintf("BACKGROUNDS:\n\n");
		consolePrintf("B: BG0 off/on: %d \n", ((~DISPLAY_CR & DISPLAY_BG0_ACTIVE)>>8)^1);
		consolePrintf("A: BG1 off/on: %d \n", ((~DISPLAY_CR & DISPLAY_BG1_ACTIVE)>>9)^1);
		consolePrintf("Y: BG2 off/on: %d \n", ((~DISPLAY_CR & DISPLAY_BG2_ACTIVE)>>10)^1);
		consolePrintf("X: BG3 off/on: %d \n\n", ((~DISPLAY_CR & DISPLAY_BG3_ACTIVE)>>11)^1);
		
		/* Shows the blend mode and blending coefficients */
		if (blend_mode == 0)
			consolePrintf("BLEND_NONE        (press start)\n\n");
		else if (blend_mode == 1)
			consolePrintf("BLEND_ALPHA       (press start)\n\n");		
		else if (blend_mode == 2)
			consolePrintf("BLEND_FADE_WHITE  (press start)\n\n");		
		else if (blend_mode == 3)
			consolePrintf("BLEND_FADE_BLACK  (press start)\n\n");
		consolePrintf("L: blend_a: %d (amount of src) \n", blend_a);
		consolePrintf("R: blend_b: %d (amount of dst) \n", blend_b);
		consolePrintf("S: blend_y: %d (b/w fade lvl)  \n", blend_y);		
	}	
	return 0;
}

